#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/socket.h>
#include <netdb.h>
#include <unixio.h>

#include <descrip.h>

#include "vms_socket.h"

int socket_open(const char *host, int port)
{
    int sd, stat;
    char sport[11];
    struct addrinfo hints, *res;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;
    hints.ai_flags = 0;
    sprintf(sport, "%d", port);
    stat = getaddrinfo(host, sport, &hints, &res);
    if(stat != 0) return -1;
    sd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
    if(sd < 0) return sd;
    stat = connect(sd, res->ai_addr, res->ai_addrlen);
    if(stat != 0) return -1;
    return sd;
}

int socket_send(int sd, const char *outmsg)
{
    int stat;
    stat = send(sd, outmsg, strlen(outmsg), 0);
    return stat;
}

int socket_recv(int sd, char *inmsg, int inmsglen)
{
    int ix, len;
    ix = 0;
    inmsg[ix] = '\0';
    while(ix < inmsglen && strstr(inmsg, "\n") == NULL)
    {
        len = recv(sd, inmsg + ix, 1, 0);
        if(len == 0) return 0;
        ix = ix + len;
        inmsg[ix] = 0;
    }
    ix--;
    while(ix >= 0 && (inmsg[ix] == '\n' || inmsg[ix] == '\r'))
    {
        inmsg[ix] = 0;
        ix--;
    }
    return ix;
}

void socket_close(int sd)
{
    close(sd);
}

static char *cstr(struct dsc$descriptor_s *s)
{
    int ix;
    char *res;
    res = malloc(s->dsc$w_length + 1);
    strncpy(res, s->dsc$a_pointer, s->dsc$w_length);
    res[s->dsc$w_length] = 0;
    /* trim traling spaces */
    ix = s->dsc$w_length - 1;
    while(ix >= 0 && res[ix] == ' ')
    {
        res[ix] = 0;
        ix--;
    }
    return res;
}

int vms_socket_open(struct dsc$descriptor_s *host, int *port)
{
    int sd;
    char *host2;
    host2 = cstr(host);
    sd = socket_open(host2, *port);
    free(host2);
    return sd;
}

int vms_socket_send(int *sd, struct dsc$descriptor_s *outmsg)
{
    int stat;
    char *outmsg2;
    outmsg2 = cstr(outmsg);
    stat = socket_send(*sd, outmsg2);
    socket_send(*sd, "\r\n"); /* add CR LF */
    free(outmsg2);
    return stat;
}

int vms_socket_recv(int *sd, struct dsc$descriptor_s *inmsg, int *inmsglen)
{
    int stat;
    stat = socket_recv(*sd, inmsg->dsc$a_pointer, inmsg->dsc$w_length);
    *inmsglen = strlen(inmsg->dsc$a_pointer);
    return stat;
}

void vms_socket_close(int *sd)
{
    socket_close(*sd);
}
